@@ -1,9 +1,10 @@
 # PyCO2SYS: marine carbonate system calculations in Python.
-# Copyright (C) 2020  Matthew Paul Humphreys et al.  (GNU GPLv3)
+# Copyright (C) 2020--2021  Matthew P. Humphreys et al.  (GNU GPLv3)
 """Carbonate system solving in N dimensions."""
 
+import itertools
 from autograd import numpy as np
-from .. import equilibria, salts, solve
+from .. import convert, equilibria, salts, solve, uncertainty
 
 # Define function input keys that should be converted to floats
 input_floats = {
@@ -36,12 +37,12 @@ input_floats = {
     "k_carbonic_2_out",
     "k_fluoride",
     "k_fluoride_out",
-    "k_phosphate_1",
-    "k_phosphate_1_out",
-    "k_phosphate_2",
-    "k_phosphate_2_out",
-    "k_phosphate_3",
-    "k_phosphate_3_out",
+    "k_phosphoric_1",
+    "k_phosphoric_1_out",
+    "k_phosphoric_2",
+    "k_phosphoric_2_out",
+    "k_phosphoric_3",
+    "k_phosphoric_3_out",
     "k_silicate",
     "k_silicate_out",
     "k_sulfide",
@@ -54,6 +55,8 @@ input_floats = {
     "k_aragonite_out",
     "fugacity_factor",
     "fugacity_factor_out",
+    "vp_factor",
+    "vp_factor_out",
     "gas_constant",
     "gas_constant_out",
     # Added in v1.6.0:
@@ -82,7 +85,7 @@ def broadcast1024(*args):
 
 def condition(args, to_shape=None):
     """Condition n-d args for PyCO2SYS.
-    
+
     If NumPy can broadcast the args together, they are a valid combination, and they
     will be combined following NumPy broadcasting rules.
 
@@ -103,7 +106,9 @@ def condition(args, to_shape=None):
             args_broadcast_shape = args_broadcast.shape
         # Broadcast the non-scalar args to a consistent shape
         args_conditioned = {
-            k: np.broadcast_to(v, args_broadcast_shape) if not np.isscalar(v) else v
+            k: np.broadcast_to(v, args_broadcast_shape)
+            if k in ["par1", "par2", "par1_type", "par2_type",] or not np.isscalar(v)
+            else v
             for k, v in args.items()
         }
         # Convert to float, where needed
@@ -118,90 +123,109 @@ def condition(args, to_shape=None):
 
 
 def _get_in_out(core, others, k_constants, suffix=""):
-    io = {
-        "pH": core["PH"],
-        "pCO2": core["PC"] * 1e6,
-        "fCO2": core["FC"] * 1e6,
-        "bicarbonate": core["HCO3"] * 1e6,
-        "carbonate": core["CARB"] * 1e6,
-        "aqueous_CO2": core["CO2"] * 1e6,
-        "alkalinity_borate": others["BAlk"] * 1e6,
-        "hydroxide": others["OH"] * 1e6,
-        "alkalinity_phosphate": others["PAlk"] * 1e6,
-        "alkalinity_silicate": others["SiAlk"] * 1e6,
-        "alkalinity_ammonia": others["NH3Alk"] * 1e6,
-        "alkalinity_sulfide": others["H2SAlk"] * 1e6,
-        "hydrogen_free": others["Hfree"] * 1e6,
-        "revelle_factor": others["Revelle"],
-        "saturation_calcite": others["OmegaCa"],
-        "saturation_aragonite": others["OmegaAr"],
-        "xCO2": others["xCO2dry"] * 1e6,
-        "pH_total": others["pHT"],
-        "pH_sws": others["pHS"],
-        "pH_free": others["pHF"],
-        "pH_nbs": others["pHN"],
-        "k_CO2": k_constants["K0"],
-        "k_carbonic_1": k_constants["K1"],
-        "k_carbonic_2": k_constants["K2"],
-        "k_water": k_constants["KW"],
-        "k_borate": k_constants["KB"],
-        "k_bisulfate": k_constants["KSO4"],
-        "k_fluoride": k_constants["KF"],
-        "k_phosphoric_1": k_constants["KP1"],
-        "k_phosphoric_2": k_constants["KP2"],
-        "k_phosphoric_3": k_constants["KP3"],
-        "k_silicate": k_constants["KSi"],
-        "k_ammonia": k_constants["KNH3"],
-        "k_sulfide": k_constants["KH2S"],
-        "k_calcite": k_constants["KCa"],
-        "k_aragonite": k_constants["KAr"],
-        "gamma_dic": others["gammaTC"],
-        "beta_dic": others["betaTC"],
-        "omega_dic": others["omegaTC"],
-        "gamma_alk": others["gammaTA"],
-        "beta_alk": others["betaTA"],
-        "omega_alk": others["omegaTA"],
-        "isocapnic_quotient": others["isoQ"],
-        "isocapnic_quotient_approx": others["isoQx"],
-        "psi": others["psi"],
-        "substrate_inhibitor_ratio": others["SIR"],
-        "fugacity_factor": k_constants["FugFac"],
-        "fH": k_constants["fH"],
+    """Assemble portion of CO2SYS_nd output dict containing input/output variables."""
+    io = {}
+    if core is not None:
+
+        def add_if_in_core(external, internal, factor=1):
+            if internal in core:
+                io.update({external: core[internal] * factor})
+
+        add_if_in_core("pH", "PH", factor=1)
+        add_if_in_core("pCO2", "PC", factor=1e6)
+        add_if_in_core("fCO2", "FC", factor=1e6)
+        add_if_in_core("bicarbonate", "HCO3", factor=1e6)
+        add_if_in_core("carbonate", "CARB", factor=1e6)
+        add_if_in_core("aqueous_CO2", "CO2", factor=1e6)
+        add_if_in_core("xCO2", "XC", factor=1e6)
+    if others is not None:
+
+        def add_if_in_others(external, internal, factor=1):
+            if internal in others:
+                io.update({external: others[internal] * factor})
+
+        add_if_in_others("alkalinity_borate", "BAlk", factor=1e6)
+        add_if_in_others("hydroxide", "OH", factor=1e6)
+        add_if_in_others("alkalinity_phosphate", "PAlk", factor=1e6)
+        add_if_in_others("alkalinity_silicate", "SiAlk", factor=1e6)
+        add_if_in_others("alkalinity_ammonia", "NH3Alk", factor=1e6)
+        add_if_in_others("alkalinity_sulfide", "H2SAlk", factor=1e6)
+        add_if_in_others("hydrogen_free", "Hfree", factor=1e6)
+        add_if_in_others("revelle_factor", "Revelle", factor=1)
+        add_if_in_others("saturation_calcite", "OmegaCa", factor=1)
+        add_if_in_others("saturation_aragonite", "OmegaAr", factor=1)
+        add_if_in_others("pH_total", "pHT", factor=1)
+        add_if_in_others("pH_sws", "pHS", factor=1)
+        add_if_in_others("pH_free", "pHF", factor=1)
+        add_if_in_others("pH_nbs", "pHN", factor=1)
+        add_if_in_others("gamma_dic", "gammaTC", factor=1)
+        add_if_in_others("beta_dic", "betaTC", factor=1)
+        add_if_in_others("omega_dic", "omegaTC", factor=1)
+        add_if_in_others("gamma_alk", "gammaTA", factor=1)
+        add_if_in_others("beta_alk", "betaTA", factor=1)
+        add_if_in_others("omega_alk", "omegaTA", factor=1)
+        add_if_in_others("isocapnic_quotient", "isoQ", factor=1)
+        add_if_in_others("isocapnic_quotient_approx", "isoQx", factor=1)
+        add_if_in_others("psi", "psi", factor=1)
+        add_if_in_others("substrate_inhibitor_ratio", "SIR", factor=1)
         # Added in v1.6.0:
-        "k_alpha": k_constants["alpha"],
-        "alkalinity_alpha": others["alk_alpha"] * 1e6,
-        "k_beta": k_constants["beta"],
-        "alkalinity_beta": others["alk_beta"] * 1e6,
-    }
-    for c in [
-        "HCO3",
-        "CO3",
-        "CO2",
-        "BOH4",
-        "BOH3",
-        "OH",
-        "Hfree",
-        "H3PO4",
-        "H2PO4",
-        "HPO4",
-        "PO4",
-        "H3SiO4",
-        "H4SiO4",
-        "NH3",
-        "NH4",
-        "HS",
-        "H2S",
-        "HSO4",
-        "SO4",
-        "HF",
-        "F",
-        "alpha",
-        "alphaH",
-        "beta",
-        "betaH",
-    ]:
-        if c in others:
-            io[c] = others[c] * 1e6
+        add_if_in_others("alkalinity_alpha", "alk_alpha", factor=1e6)
+        add_if_in_others("alkalinity_beta", "alk_beta", factor=1e6)
+        for c in [
+            "HCO3",
+            "CO3",
+            "CO2",
+            "BOH4",
+            "BOH3",
+            "OH",
+            "Hfree",
+            "H3PO4",
+            "H2PO4",
+            "HPO4",
+            "PO4",
+            "H3SiO4",
+            "H4SiO4",
+            "NH3",
+            "NH4",
+            "HS",
+            "H2S",
+            "HSO4",
+            "SO4",
+            "HF",
+            "F",
+            "alpha",
+            "alphaH",
+            "beta",
+            "betaH",
+        ]:
+            if c in others:
+                io[c] = others[c] * 1e6
+    if k_constants is not None:
+        io.update(
+            {
+                "k_CO2": k_constants["K0"],
+                "k_carbonic_1": k_constants["K1"],
+                "k_carbonic_2": k_constants["K2"],
+                "k_water": k_constants["KW"],
+                "k_borate": k_constants["KB"],
+                "k_bisulfate": k_constants["KSO4"],
+                "k_fluoride": k_constants["KF"],
+                "k_phosphoric_1": k_constants["KP1"],
+                "k_phosphoric_2": k_constants["KP2"],
+                "k_phosphoric_3": k_constants["KP3"],
+                "k_silicate": k_constants["KSi"],
+                "k_ammonia": k_constants["KNH3"],
+                "k_sulfide": k_constants["KH2S"],
+                "k_calcite": k_constants["KCa"],
+                "k_aragonite": k_constants["KAr"],
+                "fugacity_factor": k_constants["FugFac"],
+                "vp_factor": k_constants["VPFac"],
+                "fH": k_constants["fH"],
+                # Added in v1.6.0:
+                "k_alpha": k_constants["k_alpha"],
+                "k_beta": k_constants["k_beta"],
+            }
+        )
     return {"{}{}".format(k, suffix): v for k, v in io.items()}
 
 
@@ -215,40 +239,50 @@ def _get_results_dict(
     others_out,
     k_constants_out,
 ):
-    """Assemble the results dict for CO2SYS."""
-    results = {
-        "par1": args["par1"],
-        "par2": args["par2"],
-        "par1_type": args["par1_type"],
-        "par2_type": args["par2_type"],
-        "opt_k_bisulfate": args["opt_k_bisulfate"],
-        "opt_k_carbonic": args["opt_k_carbonic"],
-        "opt_k_fluoride": args["opt_k_fluoride"],
-        "opt_total_borate": args["opt_total_borate"],
-        "opt_gas_constant": args["opt_gas_constant"],
-        "opt_pH_scale": args["opt_pH_scale"],
-        "buffers_mode": args["buffers_mode"],
-        "salinity": totals["Sal"],
-        "temperature": args["temperature"],
-        "pressure": args["pressure"],
-        "total_ammonia": totals["TNH3"] * 1e6,
-        "total_borate": totals["TB"] * 1e6,
-        "total_calcium": totals["TCa"] * 1e6,
-        "total_fluoride": totals["TF"] * 1e6,
-        "total_phosphate": totals["TPO4"] * 1e6,
-        "total_silicate": totals["TSi"] * 1e6,
-        "total_sulfate": totals["TSO4"] * 1e6,
-        "total_sulfide": totals["TH2S"] * 1e6,
-        "peng_correction": totals["PengCorrection"] * 1e6,
-        "gas_constant": k_constants_in["RGas"],
-        "alkalinity": core_in["TA"] * 1e6,
-        "dic": core_in["TC"] * 1e6,
-        # Added in v1.6.0:
-        "total_alpha": totals["alpha"] * 1e6,
-        "total_beta": totals["beta"] * 1e6,
-    }
+    """Assemble the results dict for CO2SYS_nd."""
+    results = {}
+    if core_in is not None:
+        results.update(
+            {"par1": args["par1"], "par1_type": args["par1_type"],}
+        )
+        if "TA" in core_in and "TC" in core_in:
+            results.update(
+                {
+                    "par2": args["par2"],
+                    "par2_type": args["par2_type"],
+                    "alkalinity": core_in["TA"] * 1e6,
+                    "dic": core_in["TC"] * 1e6,
+                }
+            )
+    results.update(
+        {
+            "opt_k_bisulfate": args["opt_k_bisulfate"],
+            "opt_k_carbonic": args["opt_k_carbonic"],
+            "opt_k_fluoride": args["opt_k_fluoride"],
+            "opt_total_borate": args["opt_total_borate"],
+            "opt_gas_constant": args["opt_gas_constant"],
+            "opt_pH_scale": args["opt_pH_scale"],
+            "buffers_mode": args["buffers_mode"],
+            "salinity": totals["Sal"],
+            "temperature": args["temperature"],
+            "pressure": args["pressure"],
+            "total_ammonia": totals["TNH3"] * 1e6,
+            "total_borate": totals["TB"] * 1e6,
+            "total_calcium": totals["TCa"] * 1e6,
+            "total_fluoride": totals["TF"] * 1e6,
+            "total_phosphate": totals["TPO4"] * 1e6,
+            "total_silicate": totals["TSi"] * 1e6,
+            "total_sulfate": totals["TSO4"] * 1e6,
+            "total_sulfide": totals["TH2S"] * 1e6,
+            "peng_correction": totals["PengCorrection"] * 1e6,
+            "gas_constant": k_constants_in["RGas"],
+            # Added in v1.6.0:
+            "total_alpha": totals["total_alpha"] * 1e6,
+            "total_beta": totals["total_beta"] * 1e6,
+        }
+    )
     results.update(_get_in_out(core_in, others_in, k_constants_in, suffix=""))
-    if core_out is not None:
+    if "temperature_out" in args:
         results.update(
             {
                 "temperature_out": args["temperature_out"],
@@ -408,14 +442,17 @@ gradables = [
     "alphaH",
     "beta",
     "betaH",
+    # Added in v1.7.0:
+    "vp_factor",
+    "vp_factor_out",
 ]
 
 
 def CO2SYS(
-    par1,
-    par2,
-    par1_type,
-    par2_type,
+    par1=None,
+    par2=None,
+    par1_type=None,
+    par2_type=None,
     salinity=35,
     temperature=25,
     pressure=0,
@@ -450,12 +487,12 @@ def CO2SYS(
     k_carbonic_2_out=None,
     k_fluoride=None,
     k_fluoride_out=None,
-    k_phosphate_1=None,
-    k_phosphate_1_out=None,
-    k_phosphate_2=None,
-    k_phosphate_2_out=None,
-    k_phosphate_3=None,
-    k_phosphate_3_out=None,
+    k_phosphoric_1=None,
+    k_phosphoric_1_out=None,
+    k_phosphoric_2=None,
+    k_phosphoric_2_out=None,
+    k_phosphoric_3=None,
+    k_phosphoric_3_out=None,
     k_silicate=None,
     k_silicate_out=None,
     k_sulfide=None,
@@ -477,17 +514,30 @@ def CO2SYS(
     total_beta=None,
     k_beta=None,
     k_beta_out=None,
+    # Added in v1.7.0:
+    vp_factor=None,
+    vp_factor_out=None,
+    grads_of=None,
+    grads_wrt=None,
+    uncertainty_into=None,
+    uncertainty_from=None,
 ):
     """Run CO2SYS with n-dimensional args allowed."""
-    args = condition(locals())
+    args = locals()
+    keys_u = ["grads_of", "grads_wrt", "uncertainty_into", "uncertainty_from"]
+    args_set = {k: v for k, v in args.items() if k not in keys_u and v is not None}
+    args_u = {}
+    for arg in keys_u:
+        args_u[arg] = args.pop(arg)
+    args = condition(args)
     # Prepare totals dict
     totals_optional = {
         "total_borate": "TB",
         "total_calcium": "TCa",
         "total_fluoride": "TF",
         "total_sulfate": "TSO4",
-        "total_alpha": "alpha",
-        "total_beta": "beta",
+        "total_alpha": "total_alpha",
+        "total_beta": "total_beta",
     }
     if np.any(np.isin(list(args.keys()), list(totals_optional.keys()))):
         totals = {
@@ -510,6 +560,7 @@ def CO2SYS(
     # Prepare equilibrium constants dict (input conditions)
     k_constants_optional = {
         "fugacity_factor": "FugFac",
+        "vp_factor": "VPFac",
         "gas_constant": "RGas",
         "k_ammonia": "KNH3",
         "k_borate": "KB",
@@ -518,16 +569,16 @@ def CO2SYS(
         "k_carbonic_1": "K1",
         "k_carbonic_2": "K2",
         "k_fluoride": "KF",
-        "k_phosphate_1": "KP1",
-        "k_phosphate_2": "KP2",
-        "k_phosphate_3": "KP3",
+        "k_phosphoric_1": "KP1",
+        "k_phosphoric_2": "KP2",
+        "k_phosphoric_3": "KP3",
         "k_silicate": "KSi",
         "k_sulfide": "KH2S",
         "k_water": "KW",
         "k_calcite": "KCa",
         "k_aragonite": "KAr",
-        "k_alpha": "alpha",
-        "k_beta": "beta",
+        "k_alpha": "k_alpha",
+        "k_beta": "k_beta",
     }
     if np.any(np.isin(list(args.keys()), list(k_constants_optional.keys()))):
         k_constants_in = {
@@ -548,41 +599,103 @@ def CO2SYS(
         args["opt_gas_constant"],
         Ks=k_constants_in,
     )
-    # Solve the core marine carbonate system at input conditions
-    core_in = solve.core(
-        args["par1"],
-        args["par2"],
-        args["par1_type"],
-        args["par2_type"],
-        totals,
-        k_constants_in,
-        convert_units=True,
-    )
-    # Calculate the rest at input conditions
-    others_in = solve.others(
-        core_in,
-        args["temperature"],
-        args["pressure"],
-        totals,
-        k_constants_in,
-        args["opt_pH_scale"],
-        args["opt_k_carbonic"],
-        args["buffers_mode"],
-    )
+    # Solve the core marine carbonate system at input conditions, if provided
+    if par1 is not None:
+        assert par1_type is not None, "PyCO2SYS error: you must provide par1_type."
+    if par2 is not None:
+        assert par2_type is not None, "PyCO2SYS error: you must provide par2_type."
+    if par1 is not None and par2 is not None:
+        core_in = solve.core(
+            args["par1"],
+            args["par2"],
+            args["par1_type"],
+            args["par2_type"],
+            totals,
+            k_constants_in,
+            convert_units=True,
+        )
+        # Calculate the rest at input conditions
+        others_in = solve.others(
+            core_in,
+            args["temperature"],
+            args["pressure"],
+            totals,
+            k_constants_in,
+            args["opt_pH_scale"],
+            args["opt_k_carbonic"],
+            args["buffers_mode"],
+        )
+    elif par1 is not None and par2 is None:
+        core_in = {}
+        others_in = {}
+        # pH only
+        if np.any(args["par1_type"] == 3):
+            core_in.update(
+                {"PH": np.where(args["par1_type"] == 3, args["par1"], np.nan)}
+            )
+            pH_total, pH_sws, pH_free, pH_nbs = convert.pH_to_all_scales(
+                core_in["PH"], args["opt_pH_scale"], totals, k_constants_in
+            )
+            others_in.update(
+                {"pHT": pH_total, "pHS": pH_sws, "pHF": pH_free, "pHN": pH_nbs,}
+            )
+        # One of pCO2, fCO2, CO2(aq) or xCO2 only
+        if np.any(np.isin(args["par1_type"], [4, 5, 8, 9])):
+            fCO2 = (
+                np.where(
+                    args["par1_type"] == 5,
+                    args["par1"],
+                    np.where(
+                        args["par1_type"] == 4,
+                        convert.pCO2_to_fCO2(args["par1"], k_constants_in),
+                        np.where(
+                            args["par1_type"] == 8,
+                            convert.CO2aq_to_fCO2(args["par1"], k_constants_in),
+                            np.where(
+                                args["par1_type"] == 9,
+                                convert.xCO2_to_fCO2(args["par1"], k_constants_in),
+                                np.nan,
+                            ),
+                        ),
+                    ),
+                )
+                * 1e-6
+            )
+            pCO2 = np.where(
+                args["par1_type"] == 4,
+                args["par1"] * 1e-6,
+                convert.fCO2_to_pCO2(fCO2, k_constants_in),
+            )
+            CO2aq = np.where(
+                args["par1_type"] == 8,
+                args["par1"] * 1e-6,
+                convert.fCO2_to_CO2aq(fCO2, k_constants_in),
+            )
+            xCO2 = np.where(
+                args["par1_type"] == 9,
+                args["par1"] * 1e-6,
+                convert.fCO2_to_xCO2(fCO2, k_constants_in),
+            )
+            core_in.update(
+                {"PC": pCO2, "FC": fCO2, "CO2": CO2aq, "XC": xCO2,}
+            )
+    else:
+        core_in = None
+        others_in = None
     # If requested, solve the core marine carbonate system at output conditions
-    if "pressure_out" in args.keys() or "temperature_out" in args.keys():
+    if "pressure_out" in args or "temperature_out" in args:
         # Make sure we've got output values for both temperature and pressure
-        if "pressure_out" in args.keys():
-            if "temperature_out" not in args.keys():
+        if "pressure_out" in args:
+            if "temperature_out" not in args:
                 args["temperature_out"] = args["temperature"]
-        if "temperature_out" in args.keys():
-            if "pressure_out" not in args.keys():
+        if "temperature_out" in args:
+            if "pressure_out" not in args:
                 args["pressure_out"] = args["pressure"]
         # Prepare equilibrium constants dict (output conditions)
         k_constants_optional_out = {
             "{}_out".format(k): v for k, v in k_constants_optional.items()
         }
-        if np.any(np.isin(list(args.keys()), k_constants_optional_out)):
+        if np.any(np.isin(list(args.keys()), list(k_constants_optional_out.keys()))):
             k_constants_out = {
                 k_constants_optional_out[k]: v
                 for k, v in args.items()
@@ -601,32 +714,49 @@ def CO2SYS(
             args["opt_gas_constant"],
             Ks=k_constants_out,
         )
-        # Solve the core marine carbonate system at output conditions
-        core_out = solve.core(
-            core_in["TA"],
-            core_in["TC"],
-            1,
-            2,
-            totals,
-            k_constants_out,
-            convert_units=False,
-        )
-        # Calculate the rest at output conditions
-        others_out = solve.others(
-            core_out,
-            args["temperature_out"],
-            args["pressure_out"],
-            totals,
-            k_constants_out,
-            args["opt_pH_scale"],
-            args["opt_k_carbonic"],
-            args["buffers_mode"],
-        )
+        # Solve the core marine carbonate system at output conditions, if requested
+        if par1 is not None and par2 is not None:
+            core_out = solve.core(
+                core_in["TA"],
+                core_in["TC"],
+                1,
+                2,
+                totals,
+                k_constants_out,
+                convert_units=False,
+            )
+            # Calculate the rest at output conditions
+            others_out = solve.others(
+                core_out,
+                args["temperature_out"],
+                args["pressure_out"],
+                totals,
+                k_constants_out,
+                args["opt_pH_scale"],
+                args["opt_k_carbonic"],
+                args["buffers_mode"],
+            )
+        elif par1 is not None and par2 is None:
+            core_out = {}
+            others_out = {}
+            # One of pCO2, fCO2, CO2(aq) or xCO2 only
+            if np.any(np.isin(args["par1_type"], [4, 5, 8, 9])):
+                # Takahashi et al. (2009) DSR2 Eq. 2
+                core_out["PC"] = core_in["PC"] * np.exp(
+                    0.0433 * (temperature_out - temperature)
+                    - 4.35e-5 * (temperature_out ** 2 - temperature ** 2)
+                )
+                core_out["FC"] = convert.pCO2_to_fCO2(core_out["PC"], k_constants_out)
+                core_out["CO2"] = convert.fCO2_to_CO2aq(core_out["FC"], k_constants_out)
+                core_out["XC"] = convert.fCO2_to_xCO2(core_out["FC"], k_constants_out)
+        else:
+            core_out = None
+            others_out = None
     else:
         core_out = None
         others_out = None
         k_constants_out = None
-    return _get_results_dict(
+    results = _get_results_dict(
         args,
         totals,
         core_in,
@@ -636,3 +766,138 @@ def CO2SYS(
         others_out,
         k_constants_out,
     )
+    # Do uncertainty propagation, if requested
+    if grads_of is not None and grads_wrt is not None:
+        forward = uncertainty.forward_nd(results, grads_of, grads_wrt, **args_set)[0]
+        grads = {}
+        for of, wrt in itertools.product(grads_of, grads_wrt):
+            grads["d_{}__d_{}".format(of, wrt)] = forward[of][wrt]
+        results.update(grads)
+    if uncertainty_into is not None and uncertainty_from is not None:
+        uncertainties, components = uncertainty.propagate_nd(
+            results, uncertainty_into, uncertainty_from, **args_set
+        )
+        uncerts = {}
+        for into in uncertainty_into:
+            uncerts["u_{}".format(into)] = uncertainties[into]
+            for ufrom in uncertainty_from:
+                ufrom = ufrom.split("__f")[0]
+                uncerts["u_{}__{}".format(into, ufrom)] = components[into][ufrom]
+        results.update(uncerts)
+    return results
+
+
+def assemble(
+    salinity=35,
+    temperature=25,
+    pressure=0,
+    temperature_out=None,
+    pressure_out=None,
+    total_ammonia=0,
+    total_phosphate=0,
+    total_silicate=0,
+    total_sulfide=0,
+    total_borate=None,
+    total_calcium=None,
+    total_fluoride=None,
+    total_sulfate=None,
+    opt_gas_constant=3,
+    opt_k_bisulfate=1,
+    opt_k_carbonic=16,
+    opt_k_fluoride=1,
+    opt_pH_scale=1,
+    opt_total_borate=1,
+    k_ammonia=None,
+    k_borate=None,
+    k_bisulfate=None,
+    k_CO2=None,
+    k_carbonic_1=None,
+    k_carbonic_2=None,
+    k_fluoride=None,
+    k_phosphoric_1=None,
+    k_phosphoric_2=None,
+    k_phosphoric_3=None,
+    k_silicate=None,
+    k_sulfide=None,
+    k_water=None,
+    k_calcite=None,
+    k_aragonite=None,
+    fugacity_factor=None,
+    vp_factor=None,
+    gas_constant=None,
+    gas_constant_out=None,
+    total_alpha=None,
+    k_alpha=None,
+    total_beta=None,
+    k_beta=None,
+):
+    args = condition(locals())
+    # Prepare totals dict
+    totals_optional = {
+        "total_borate": "TB",
+        "total_calcium": "TCa",
+        "total_fluoride": "TF",
+        "total_sulfate": "TSO4",
+        "total_alpha": "total_alpha",
+        "total_beta": "total_beta",
+    }
+    if np.any(np.isin(list(args.keys()), list(totals_optional.keys()))):
+        totals = {
+            totals_optional[k]: v * 1e-6
+            for k, v in args.items()
+            if k in totals_optional
+        }
+    else:
+        totals = None
+    totals = salts.assemble(
+        args["salinity"],
+        args["total_silicate"],
+        args["total_phosphate"],
+        args["total_ammonia"],
+        args["total_sulfide"],
+        args["opt_k_carbonic"],
+        args["opt_total_borate"],
+        totals=totals,
+    )
+    # Prepare equilibrium constants dict (input conditions)
+    k_constants_optional = {
+        "fugacity_factor": "FugFac",
+        "vp_factor": "VPFac",
+        "gas_constant": "RGas",
+        "k_ammonia": "KNH3",
+        "k_borate": "KB",
+        "k_bisulfate": "KSO4",
+        "k_CO2": "K0",
+        "k_carbonic_1": "K1",
+        "k_carbonic_2": "K2",
+        "k_fluoride": "KF",
+        "k_phosphoric_1": "KP1",
+        "k_phosphoric_2": "KP2",
+        "k_phosphoric_3": "KP3",
+        "k_silicate": "KSi",
+        "k_sulfide": "KH2S",
+        "k_water": "KW",
+        "k_calcite": "KCa",
+        "k_aragonite": "KAr",
+        "k_alpha": "k_alpha",
+        "k_beta": "k_beta",
+    }
+    if np.any(np.isin(list(args.keys()), list(k_constants_optional.keys()))):
+        k_constants = {
+            k_constants_optional[k]: v
+            for k, v in args.items()
+            if k in k_constants_optional
+        }
+    else:
+        k_constants = None
+    k_constants = equilibria.assemble(
+        args["temperature"],
+        args["pressure"],
+        totals,
+        args["opt_pH_scale"],
+        args["opt_k_carbonic"],
+        args["opt_k_bisulfate"],
+        args["opt_k_fluoride"],
+        args["opt_gas_constant"],
+        Ks=k_constants,
+    )
