@@ -1,5 +1,5 @@
 # PyCO2SYS: marine carbonate system calculations in Python.
-# Copyright (C) 2020  Matthew Paul Humphreys et al.  (GNU GPLv3)
+# Copyright (C) 2020--2021  Matthew P. Humphreys et al.  (GNU GPLv3)
 """Evaluate residuals for TA-pH solvers."""
 
 from autograd import numpy as np
@@ -8,7 +8,7 @@ from . import get
 
 
 # Set whether to use the approximate slopes [True] or exact (Autograd) slopes [False]
-USE_APPROX = False
+use_approximate_slopes = False
 
 
 def _pHfromTATC_r(pH, TA, TC, totals, Ks):
@@ -23,9 +23,9 @@ _pHfromTATC_s = egrad(_pHfromTATC_r)
 def _pHfromTATC_s_approx(pH, TA, TC, totals, Ks):
     """Calculate residual alkalinity slope from pH and TC for solver `pHfromTATC`
     approximately, without using Autograd.
-    
+
     This is the original equation from CO2SYS for MATLAB.  It is not used in PyCO2SYS.
-    
+
     Based on CalculatepHfromTATC, version 04.01, 10-13-96, by Ernie Lewis.
     """
     K1 = Ks["K1"]
@@ -43,20 +43,14 @@ def _pHfromTATC_s_approx(pH, TA, TC, totals, Ks):
     )
 
 
-if USE_APPROX:
-
-    def pHfromTATC(pH, TA, TC, totals, Ks):
-        """Calculate delta-pH from pH and TC for solver `pHfromTATC`."""
+def pHfromTATC(pH, TA, TC, totals, Ks):
+    """Calculate delta-pH from pH and TC for solver `pHfromTATC`."""
+    if use_approximate_slopes:
         return -(
             _pHfromTATC_r(pH, TA, TC, totals, Ks)
             / _pHfromTATC_s_approx(pH, TA, TC, totals, Ks)
         )
-
-
-else:
-
-    def pHfromTATC(pH, TA, TC, totals, Ks):
-        """Calculate delta-pH from pH and TC for solver `pHfromTATC`."""
+    else:
         return -(
             _pHfromTATC_r(pH, TA, TC, totals, Ks)
             / _pHfromTATC_s(pH, TA, TC, totals, Ks)
@@ -75,9 +69,9 @@ _pHfromTAfCO2_s = egrad(_pHfromTAfCO2_r)
 def _pHfromTAfCO2_s_approx(pH, TA, fCO2, totals, Ks):
     """Calculate residual alkalinity slope from pH and fCO2 for solver `pHfromTAfCO2`
     approximately, without using Autograd.
-    
+
     This is the original equation from CO2SYS for MATLAB.  It is not used in PyCO2SYS.
-    
+
     Based on CalculatepHfromTAfCO2, version 04.01, 10-13-97, by Ernie Lewis.
     """
     K0 = Ks["K0"]
@@ -92,20 +86,14 @@ def _pHfromTAfCO2_s_approx(pH, TA, fCO2, totals, Ks):
     return np.log(10) * (HCO3 + 4 * CO3 + BAlk * H / (KB + H) + OH + H)
 
 
-if USE_APPROX:
-
-    def pHfromTAfCO2(pH, TA, fCO2, totals, Ks):
-        """Calculate delta-pH from pH and fCO2 for solver `pHfromTAfCO2`."""
+def pHfromTAfCO2(pH, TA, fCO2, totals, Ks):
+    """Calculate delta-pH from pH and fCO2 for solver `pHfromTAfCO2`."""
+    if use_approximate_slopes:
         return -(
             _pHfromTAfCO2_r(pH, TA, fCO2, totals, Ks)
             / _pHfromTAfCO2_s_approx(pH, TA, fCO2, totals, Ks)
         )
-
-
-else:
-
-    def pHfromTAfCO2(pH, TA, fCO2, totals, Ks):
-        """Calculate delta-pH from pH and fCO2 for solver `pHfromTAfCO2`."""
+    else:
         return -(
             _pHfromTAfCO2_r(pH, TA, fCO2, totals, Ks)
             / _pHfromTAfCO2_s(pH, TA, fCO2, totals, Ks)
@@ -124,9 +112,9 @@ _pHfromTACarb_s = egrad(_pHfromTACarb_r)
 def _pHfromTACarb_s_approx(pH, TA, CARB, totals, Ks):
     """Calculate residual alkalinity slope from pH and CARB for solver `pHfromTACarb`
     approximately, without using Autograd.
-    
+
     This is the original equation from CO2SYS for MATLAB.  It is not used in PyCO2SYS.
-    
+
     Based on CalculatepHfromTACarb, version 01.0, 06-12-2019, by Denis Pierrot.
     """
     K2 = Ks["K2"]
@@ -137,20 +125,14 @@ def _pHfromTACarb_s_approx(pH, TA, CARB, totals, Ks):
     return np.log(10) * (-CARB * H / K2 + BAlk * H / (KB + H) + OH + H)
 
 
-if USE_APPROX:
-
-    def pHfromTACarb(pH, TA, CARB, totals, Ks):
-        """Calculate delta-pH from pH and CARB for solver `pHfromTACarb`."""
+def pHfromTACarb(pH, TA, CARB, totals, Ks):
+    """Calculate delta-pH from pH and CARB for solver `pHfromTACarb`."""
+    if use_approximate_slopes:
         return -(
             _pHfromTACarb_r(pH, TA, CARB, totals, Ks)
             / _pHfromTACarb_s_approx(pH, TA, CARB, totals, Ks)
         )
-
-
-else:
-
-    def pHfromTACarb(pH, TA, CARB, totals, Ks):
-        """Calculate delta-pH from pH and CARB for solver `pHfromTACarb`."""
+    else:
         return -(
             _pHfromTACarb_r(pH, TA, CARB, totals, Ks)
             / _pHfromTACarb_s(pH, TA, CARB, totals, Ks)
@@ -169,7 +151,7 @@ _pHfromTAHCO3_s = egrad(_pHfromTAHCO3_r)
 def _pHfromTAHCO3_s_approx(pH, TA, HCO3, totals, Ks):
     """Calculate residual alkalinity slope from pH and HCO3 for solver `pHfromTAHCO3`
     approximately, without using Autograd.
-    
+
     This is what the original equation would have been if it were in CO2SYS for MATLAB.
     It is not used in PyCO2SYS.
     """
@@ -181,20 +163,14 @@ def _pHfromTAHCO3_s_approx(pH, TA, HCO3, totals, Ks):
     return np.log(10) * (2 * HCO3 * K2 / H + BAlk * H / (KB + H) + OH + H)
 
 
-if USE_APPROX:
-
-    def pHfromTAHCO3(pH, TA, HCO3, totals, Ks):
-        """Calculate delta-pH from pH and HCO3 for solver `pHfromTAHCO3`."""
+def pHfromTAHCO3(pH, TA, HCO3, totals, Ks):
+    """Calculate delta-pH from pH and HCO3 for solver `pHfromTAHCO3`."""
+    if use_approximate_slopes:
         return -(
             _pHfromTAHCO3_r(pH, TA, HCO3, totals, Ks)
             / _pHfromTAHCO3_s_approx(pH, TA, HCO3, totals, Ks)
         )
-
-
-else:
-
-    def pHfromTAHCO3(pH, TA, HCO3, totals, Ks):
-        """Calculate delta-pH from pH and HCO3 for solver `pHfromTAHCO3`."""
+    else:
         return -(
             _pHfromTAHCO3_r(pH, TA, HCO3, totals, Ks)
             / _pHfromTAHCO3_s(pH, TA, HCO3, totals, Ks)
